/* moz 1.0 */
const OLD_WSTRING_CONTRACTID = "@mozilla.org/supports-wstring;1";
const OLD_SUPPORTS_WSTRING = Components.interfaces.nsISupportsWString;

/* moz 1.1 */
const NEW_WSTRING_CONTRACTID = "@mozilla.org/supports-string;1";
const NEW_SUPPORTS_WSTRING = Components.interfaces.nsISupportsString;

function LJConnection (postURL, username, password, hpassword) {
  if (postURL) {
    this._init(postURL, username, password, hpassword);
  }
}

LJConnection.prototype = {
  _init : function(postURL, username, password, hpassword) {
    this.postURL = postURL;
    this.username = username;

    /* the hpassword is always assumed to be correct */
    if (hpassword) {
      this.hpassword = hpassword;
    } else {
      this.hpassword = hexMD5(password);
    }
  },

  login : function(lastmoodid, clientversion) { },
  postevent : function(event, subject, date, time, usejournal,
    mood, music, pickw, autoformat, allowcomments, backdate) { }
}

/* HTTP POST interface */
function LJHTTPPostConnection (postURL, username, password, hpassword) {
  if (postURL) {
    this._init(postURL, username, password, hpassword);
  }
}

LJHTTPPostConnection.prototype = new LJConnection();

LJHTTPPostConnection.prototype = {
  _init : function(postURL, username, password, hpassword) {
    this.postURL = postURL;
    this.username = username;

    /* the hpassword is always assumed to be correct */
    if (hpassword) {
      this.hpassword = hpassword;
    } else {
      this.hpassword = hexMD5(password);
    }
  },

  login : function(lastmoodid, clientversion) { 
    var body = "mode=login";
    body += "&ver=1";
    body += "&user=" + escape(this.username);
    body += "&hpassword=" + escape(this.hpassword);
    body += "&getpickws=1";
    body += "&getpickwurls=1";
    body += "&getmoods=" + escape(lastmoodid);
    if (clientversion) {
      body += "&clientversion=" + escape(clientversion);
    }

    var response = sendRequest(body, this.postURL);

    if (getResponseData("success", response) == "OK") {
      var result = [];
      var i;

      result.usejournal = [];
      for (i = 1; i <= getResponseData("access_count", response); i++) {
        result.usejournal[i] = getResponseData("access_" + i, response);
      }

      result.pickw = [];
      for (i = 1; i <= getResponseData("pickw_count", response); i++) {
        result.pickw[i] = [];
        result.pickw[i].keyword = getResponseData("pickw_" + i, response);
        result.pickw[i].url = getResponseData("pickwurl_" + i, response);
      }

      result.moods = [];
      for (i = 1; i <= getResponseData("mood_count", response); i++) {
        result.moods[i] = [];
        result.moods[i].id = getResponseData("mood_" + i + "_id", response);
        result.moods[i].name = getResponseData("mood_" + i + "_name", response);
      }

      return result;
    } else {
    }

    if (getResponseData("message", response)) {
      alert(getResponseData("message", response));
    }
  
    return null;
  },

  postevent : function(subject, event, date, time, usejournal,
    mood, custom, music, pickw, autoformat, allowcomments, security, backdate) {

    var year = /^(\d+)\//.exec(date)[1];
    var month = /\/(\d+)\//.exec(date)[1];
    var day = /\/(\d+)$/.exec(date)[1];

    var hour = /^(\d+):/.exec(time)[1];
    var min = /:(\d+)$/.exec(time)[1];

    var body = "mode=postevent";
    body += "&ver=1";
    body += "&user=" + escape(this.username);
    if (this.username != usejournal) {
      body += "&usejournal=" + escape(usejournal);
    }
    body += "&hpassword=" + escape(this.hpassword);

    body += "&subject=" + escape(subject);
    body += "&event=" + escape(event);

    body += "&year=" + escape(year);
    body += "&mon=" + escape(month);
    body += "&day=" + escape(day);
    body += "&hour=" + escape(hour);
    body += "&min=" + escape(min);

    body += "&prop_picture_keyword=" + escape(pickw);

    /* If mood is not composed of digits, its a moodid */
    if ((mood[0]>'0')&&(mood[0]<='9')) {
      body += "&prop_current_moodid=" + escape(mood);
    } else {
      body += "&prop_current_mood=" + escape(custom);
    }

    body += "&prop_current_music=" + escape(music);

    body += "&security=" + escape(security);
    if (security == "usemask") {
      body += "&allowmask=" + escape("1");
    }

    body += "&prop_opt_backdated=" + escape(backdate);
    body += "&prop_opt_preformatted=" + escape(autoformat);
    body += "&prop_opt_nocomments=" + escape(allowcomments);

    var response = sendRequest(body, this.postURL);

    if (getResponseData("success", response) == "OK") {
      return response;
    } else {
      return null;
    }
  }
}

function sendRequest(data, URL) {
  var body;
  try {
    body = Components.classes[OLD_WSTRING_CONTRACTID];
    body = body.createInstance(OLD_SUPPORTS_WSTRING);
  } catch (e) {
    body = Components.classes[NEW_WSTRING_CONTRACTID];
    body = body.createInstance(Components.interfaces.nsISupportsString);
  }

  body.data = data;

  var request = new XMLHttpRequest();
  request.open("POST", URL, false);
  request.send(body);

  return request.responseText;
}

function getResponseData(heading, responseText) {
  var i = 0;
  var data = null;
  var responseData = responseText.split("\n");

  while (i != responseData.length) {
    if (responseData[i] == heading) {
      data = responseData[i + 1];
      return data;
    }
    i++;
  }

  return null;
}

